//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <ConIO.H>

#define CONSOLE_RES_CLOSE   1
#define CONSOLE_RES_OK      2
#define CONSOLE_RES_ERROR   3
#define CONSOLE_RES_UNKNOWN 4

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool OpenConsole(void)
{
    if(AllocConsole())
    {
        Console_Thread_Handle = CreateThread(NULL, 0, Console_Thread, (LPVOID)0, 0,&Console_Thread_ID);
        gbConsoleOpen = true;
        return true;
    }
    return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CloseConsole(void)
{
    if(FreeConsole())
    {
        gbConsoleOpen = false;
        TerminateThread(Console_Thread, 0);
        CloseHandle(Console_Thread_Handle);
        return true;
    }
    return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int WriteConEx(char *Buffer, int Length)
{
    int RPos = 0;
    while(RPos < Length)
        _putch(Buffer[RPos++]);
    return Length;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int WriteCon(char *Buffer)
{
    return WriteConEx(Buffer, strlen(Buffer));
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int ProcessConsoleCommand(char *Command)
{
    char TempText[1024];

    if(strcmpi(Command, "ver") == 0)
    {
        sprintf(TempText, " : %s - %s (Built: %s on %s)", TitleCaption, FileVersion, __DATE__, __TIME__);
        WriteCon(TempText);
        return CONSOLE_RES_OK;
    }
    else if(strcmpi(Command, "connections") == 0)
    {
        sprintf(TempText, " : Class: %d, Global: %d, Total: %d ", MyServer.Connections, glConnections, glTotalConnects);
        WriteCon(TempText);
        return CONSOLE_RES_OK;
    }
    else{
        WriteCon(" unknown command.");
        return CONSOLE_RES_UNKNOWN;
    }

    return CONSOLE_RES_ERROR;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

DWORD WINAPI Console_Thread(LPVOID lpData)
{
    char TempText[1024];
    char Command[255];
    int CommandPos = 0;

    sprintf(TempText, "%s  [Version %s]\n\n", TitleCaption, FileVersion);
    WriteCon(TempText);

    WriteCon("> ");
    while(1)
    {
        bool WriteChar = true;
        int Key = getch();

        //ErrorMessage(NULL, IntChar(Key));

        if(Key == 8) // Backspace
        {
            CommandPos--;
            _putch(8);
            _putch(' ');
            _putch(8);
            WriteChar = false;
        }

        if(Key == 13) // Enter
        {
            if(CommandPos > 0)
            {
                Command[CommandPos] = '\0';
                int Result = ProcessConsoleCommand(Command);

                if(Result == CONSOLE_RES_CLOSE)
                {
                    return 0;
                }

                CommandPos = 0;
            }
            WriteCon("\n> ");
            WriteChar = false;
        }

        if(WriteChar)
        {
            _putch(Key);
            Command[CommandPos] = Key;
            CommandPos++;
        }

        Sleep(1);
    }

	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

