////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

SQLINTEGER CharTrim(SQLCHAR *Data, SQLINTEGER Length)
{
    SQLINTEGER Pos = Length;

    if(Length == 0)
        return Length;

    if(Data[Length-1] != ' ')
        return Length;

    Pos--;

    while(Pos != 0 && Data[Pos] == ' ')
        Pos--;

    if(Data[Pos] != ' ')
        Pos++;

    Data[Pos] = '\0';

    return Pos;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FreeSingleRowDataColumbs(struct _SQLImportMembers *SIM, int ColumnCount)
{
    int ColumbPos = 0;
    while(ColumbPos < ColumnCount)
    {
        free(SIM->SingleRowData[ColumbPos]);
        SIM->SingleRowData[ColumbPos] = NULL;
        ColumbPos++;
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FreeSingleRowData(struct _SQLImportMembers *SIM)
{
    free(SIM->SingleRowData);
    SIM->SingleRowData = NULL;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool ImportSQLResults(int Client, char *InFileName)
{
    char StatusText[255];
    char SelectStatement[255];

    SQLINTEGER outDataLength = 0;
	SQLINTEGER outRowCount = 0;

    FILE *SourceHandle = NULL;

    _SQLImportMembers SIM;

    HSTMT StatementHandle = NULL;

    sprintf(StatusText, "(%d) Importing client data.", MyServer.ClientID[Client]);
    WriteLog(StatusText);

    if(!CI[Client].WPCustDB.DBConnect("{SQL Server}", "NetBase", "sa", "SQL840823110", CI[Client].CompanyDB))
    {
        // FIXME: Need to dump the connection variables that failed to log file
        sprintf(StatusText, "(%d) Error in ImportSQLResultsFailed:: DBConnect error.", MyServer.ClientID[Client]);
        WriteLog(StatusText);
        glErrorCount++;
        return false;
    }

    if((SourceHandle = fopen(InFileName, "rb")) == NULL)
    {
        // FIXME: Need to dump the file name to log

        sprintf(StatusText, "(%d) Error in ImportSQLResultsFailed:: TargetFile open error.", MyServer.ClientID[Client]);
        WriteLog(StatusText);
        glErrorCount++;
        return false;
    }

    int TotalLengthOfAllColumbNames = 0; // For better memory management;
    int TotalLengthOfAllRowData = 0; // For better memory management
    int ImportTableNameLen = 0;
    SQLSMALLINT ColumnCount = 0;
    SQLINTEGER RowCount = 0;
    char ImportTable[256];

    fread(&ImportTableNameLen, sizeof(ImportTableNameLen), 1, SourceHandle); // Read import table name len
    fread(ImportTable, sizeof(char), ImportTableNameLen, SourceHandle); // Read import table name
    fread(&ColumnCount, sizeof(ColumnCount), 1, SourceHandle); // Read number of columbs
    fread(&RowCount, sizeof(RowCount), 1, SourceHandle);       // Read number of rows

    ImportTable[ImportTableNameLen] = '\0';

//    char UniuqeKeyName[5][64]; // Contains the names of the uniuqe columbs
//    int UniuqeKeyCol[5];       // Contains the columb position of the uniuqe columbs
//    int UniuqeKeysNeeded = 2;  // The ammount of uniuqe keys we have to find
//    int UniuqeKeysFound = 0;   // The ammount of uniuqe keys we have found

    char UniuqeKeyName[5][65];        // Contains the names of the uniuqe columbs
    int UniuqeKeyCol[5];              // Contains the columb position of the uniuqe columbs
    SQLINTEGER UniuqeKeysNeeded = 0;  // The ammount of uniuqe keys we have to find
    SQLINTEGER UniuqeKeysLoop = 0;    // Just used to loop through and get the data
    SQLINTEGER UniuqeKeysFound = 0;   // The ammount of uniuqe keys we have found

    sprintf(SelectStatement, "SELECT UniqueKeys, IKey1, IKey2, IKey3, IKey4, IKey5 FROM ImportKeys WHERE ImportTable = '%s'", ImportTable);
    if(!CI[Client].WPIndexDB.DBExecute(SelectStatement, &StatementHandle))
    {
        // FIXME: Need to dump the statement that failed to a seperate log file. (SelectStatement)

        sprintf(StatusText, "(%d) Error in ImportSQLResultsFailed:: DBExecute failed on ImportKeys.", MyServer.ClientID[Client]);
        WriteLog(StatusText);

        WriteLog(SelectStatement); // Debug Code

        if(SourceHandle) fclose(SourceHandle);
        glErrorCount++;
        return false;
    }

    if(SQLRowCount(StatementHandle, &outRowCount) != SQL_SUCCESS)
    {
        sprintf(StatusText, "(%d) Error in ImportSQLResultsFailed:: SQLRowCount failed on ImportKeys.", MyServer.ClientID[Client]);
        WriteLog(StatusText);

        DBCloseCursor(StatementHandle);

        if(SourceHandle) fclose(SourceHandle);
        glErrorCount++;
        return false;
    }

    if(outRowCount != 1)
    {
        sprintf(StatusText, "(%d) Error in ImportSQLResultsFailed:: No or more than one uniuqe keys indexed for table: '%s'. Rows: %d.", MyServer.ClientID[Client], ImportTable, outRowCount);
        WriteLog(StatusText);

        DBCloseCursor(StatementHandle);

        if(SourceHandle) fclose(SourceHandle);
        glErrorCount++;
        return false;
    }

    if(!DBFetch(StatementHandle))
    {
        sprintf(StatusText, "(%d) Error in ImportSQLResultsFailed:: DBFetch failed on ImportKeys.", MyServer.ClientID[Client]);
        WriteLog(StatusText);

        DBCloseCursor(StatementHandle);

        if(SourceHandle) fclose(SourceHandle);
        glErrorCount++;
        return false;
    }

    if(!DBGetData(StatementHandle, 1, SQL_C_LONG, &UniuqeKeysNeeded, sizeof(UniuqeKeysNeeded), &outDataLength))
    {
        sprintf(StatusText, "(%d) Error in ImportSQLResultsFailed:: DBGetData failed on UniuqeKeysNeeded.", MyServer.ClientID[Client]);
        WriteLog(StatusText);

        DBCloseCursor(StatementHandle);

        if(SourceHandle) fclose(SourceHandle);
        glErrorCount++;
        return false;
    }

    if(UniuqeKeysNeeded <= 0 || UniuqeKeysNeeded > 5)
    {
        sprintf(StatusText, "(%d) Error in ImportSQLResultsFailed:: No or too many uniuqe keys specified for table: '%s'.", MyServer.ClientID[Client], ImportTable);
        WriteLog(StatusText);

        DBCloseCursor(StatementHandle);

        if(SourceHandle) fclose(SourceHandle);
        glErrorCount++;
        return false;
    }

    // Get the names of all the uniuqe keys (Fill them in the UniuqeKeyName[] array)
    while(UniuqeKeysLoop < UniuqeKeysNeeded)
    {
        if(!DBGetData(StatementHandle, UniuqeKeysLoop+2, SQL_C_CHAR, UniuqeKeyName[UniuqeKeysLoop], 64, &outDataLength))
        {
            sprintf(StatusText, "(%d) Error in ImportSQLResultsFailed:: DBGetData failed on UniuqeKeyName.", MyServer.ClientID[Client]);
            WriteLog(StatusText);

            DBCloseCursor(StatementHandle);

            if(SourceHandle) fclose(SourceHandle);
            glErrorCount++;
            return false;
        }

        //CharTrim(UniuqeKeyName[UniuqeKeysLoop], outDataLength);
        UniuqeKeysLoop++;
    }

    DBCloseCursor(StatementHandle);

    SIM.ColumbNames = (char **) calloc(sizeof(char *), ColumnCount + 1);

    SQLSMALLINT ColNameLen = 0;

    int ColumbPos = 0;
    while(ColumbPos < ColumnCount)
    {
        fread(&ColNameLen, sizeof(ColNameLen), 1, SourceHandle);
        SIM.ColumbNames[ColumbPos] = (char *) calloc(sizeof(char), ColNameLen + 1);
        fread(SIM.ColumbNames[ColumbPos], sizeof(char), ColNameLen, SourceHandle);
        SIM.ColumbNames[ColumbPos][ColNameLen] = '\0';
        TotalLengthOfAllColumbNames = (TotalLengthOfAllColumbNames + ColNameLen);

        // Is this columb one of the uniuqe keys
        if(UniuqeKeysFound < UniuqeKeysNeeded)
        {
            int UniuqeKeyCount = 0;
            while(UniuqeKeyCount < UniuqeKeysNeeded)
            {
                if(strcmpi(UniuqeKeyName[UniuqeKeyCount], SIM.ColumbNames[ColumbPos]) == 0)
                {
                    UniuqeKeyCol[UniuqeKeyCount] = ColumbPos;
                    UniuqeKeysFound++;
                }
                UniuqeKeyCount++;
            }
        }

        ColumbPos++;
    }
/*
    ColumbPos = 0;
    while(ColumbPos < ColumnCount)
    {
        free(SIM.ColumbNames[ColumbPos]);
        SIM.ColumbNames[ColumbPos] = NULL;
        ColumbPos++;
    }
    free(SIM.ColumbNames);
    SIM.ColumbNames = NULL;
*/
/* // Debug code
    ColumbPos = 0;
    while(ColumbPos < ColumnCount)
    {
        printf("SIM.ColumbNames[%d]:%s\n", ColumbPos, SIM.ColumbNames[ColumbPos]);
        ColumbPos++;
    }

    printf("UniuqeKeysFound: %d\n", UniuqeKeysFound);
*/

	int UniuqeKeyCount = 0;
    while(UniuqeKeysNeeded < UniuqeKeysFound)
    {
        sprintf(StatusText, "(%d) Error in ImportSQLResultsFailed:: Required uniuqe keys wer not found for: '%s'.", MyServer.ClientID[Client], ImportTable);
        WriteLog(StatusText);

        FreeSingleRowDataColumbs(&SIM, ColumnCount);

        if(SourceHandle) fclose(SourceHandle);
        glErrorCount++;
        return false;
    }

    sprintf(StatusText, "(%d) Starting row by row data import.", MyServer.ClientID[Client]);
    WriteLog(StatusText);

    SIM.SingleRowData = (char **) calloc(sizeof(char *), ColumnCount + 1);

    SQLINTEGER RowDataLen = 0;

    int RowPos = 0;
    while(RowPos < RowCount)
    {
        int BaseMemoryRequirement = 0;
        TotalLengthOfAllRowData = 0;

        // Get row data
        ColumbPos = 0;
        while(ColumbPos < ColumnCount)
        {
            fread(&RowDataLen, sizeof(RowDataLen), 1, SourceHandle);
            SIM.SingleRowData[ColumbPos] = (char *) calloc(sizeof(char), RowDataLen + 1);
            fread(SIM.SingleRowData[ColumbPos], sizeof(char), RowDataLen, SourceHandle);
            SIM.SingleRowData[ColumbPos][RowDataLen] = '\0';
            TotalLengthOfAllRowData = (TotalLengthOfAllRowData + RowDataLen);
            ColumbPos++;
        }

        BaseMemoryRequirement = ((TotalLengthOfAllColumbNames + TotalLengthOfAllRowData) + (ColumnCount * 5)) + 1024;

        char WhereStatement[1024];
        char CheckExistanceStatement[1024];

        // Assemble the where statement from the row data, the uniuqe key names and positions
        if(UniuqeKeysFound > 0)
        {
            // Assemble the where statement
            sprintf(WhereStatement, "WHERE %s = %s", UniuqeKeyName[0], SIM.SingleRowData[UniuqeKeyCol[0]]);
        	int UniuqeKeyCount = 1;
            while(UniuqeKeyCount < UniuqeKeysFound)
            {
                strcat(WhereStatement, " AND ");
                strcat(WhereStatement, UniuqeKeyName[UniuqeKeyCount]);
                strcat(WhereStatement, " = ");
                strcat(WhereStatement, SIM.SingleRowData[UniuqeKeyCol[UniuqeKeyCount]]);
                UniuqeKeyCount++;
            }

            // Assemble the check existance statement
            sprintf(CheckExistanceStatement, "SELECT %s", UniuqeKeyName[0]);
            strcat(CheckExistanceStatement, " FROM ");
            strcat(CheckExistanceStatement, ImportTable);
            strcat(CheckExistanceStatement, " ");
            strcat(CheckExistanceStatement, WhereStatement);

            //printf("CheckExistanceStatement: %s\n", CheckExistanceStatement);
            //printf("WhereStatement: %s\n", WhereStatement);
        }

        if(!CI[Client].WPCustDB.DBExecute(CheckExistanceStatement, &StatementHandle))
        {
            // FIXME: Need to dump the statement that failed to a seperate log file

            sprintf(StatusText, "(%d) Error in ImportSQLResultsFailed:: DBExecute failed on CheckExistanceStatement.", MyServer.ClientID[Client]);
            WriteLog(StatusText);

            FreeSingleRowDataColumbs(&SIM, ColumnCount);
            FreeSingleRowData(&SIM);

            if(SourceHandle) fclose(SourceHandle);
            glErrorCount++;
            return false;
        }

        SQLINTEGER ReturnedRowCount = 0;
        if(SQLRowCount(StatementHandle, &ReturnedRowCount) != SQL_SUCCESS)
        {
            // FIXME: Need to dump the statement that returned these rows seperate log file. (CheckExistanceStatement)

            sprintf(StatusText, "(%d) Error in ImportSQLResultsFailed:: Failed to retrieve row count.", MyServer.ClientID[Client]);
            WriteLog(StatusText);
            DBCloseCursor(&StatementHandle);

            FreeSingleRowDataColumbs(&SIM, ColumnCount);
            FreeSingleRowData(&SIM);

            if(SourceHandle) fclose(SourceHandle);
            glErrorCount++;
            return false;
        }

        DBCloseCursor(&StatementHandle);

        char *ImportStatement = NULL;

        //----------------------------------------------------------------------------------------------
        if(ReturnedRowCount == 0) // Do insert
        {
            ImportStatement = (char *) calloc( sizeof(char), BaseMemoryRequirement + 1);

//            printf("Inserting row %d. Memory: %d\n", RowPos, BaseMemoryRequirement);

            sprintf(ImportStatement, "INSERT INTO %s (", ImportTable);
    
            ColumbPos = 0;
            while(ColumbPos < ColumnCount)
            {
                strcat(ImportStatement, SIM.ColumbNames[ColumbPos]);
                if(ColumbPos < ColumnCount - 1)
                {
                    strcat(ImportStatement, ",");
                }
                ColumbPos++;
            }
            strcat(ImportStatement, ") VALUES(");

            ColumbPos = 0;
            while(ColumbPos < ColumnCount)
            {
                strcat(ImportStatement, SIM.SingleRowData[ColumbPos]);
                if(ColumbPos < ColumnCount - 1)
                {
                    strcat(ImportStatement, ",");
                }
                ColumbPos++;
            }
            strcat(ImportStatement, ") ");

//          ErrorMessage(NULL, ImportStatement); // Debug code
        }
        //----------------------------------------------------------------------------------------------
        else { // Do update
            BaseMemoryRequirement = (BaseMemoryRequirement + strlen(WhereStatement));
            ImportStatement = (char *) calloc( sizeof(char), BaseMemoryRequirement + 1);

//            printf("Updating row %d. Memory: %d\n", RowPos, BaseMemoryRequirement);

            sprintf(ImportStatement, "UPDATE %s SET ", ImportTable);

            ColumbPos = 0;
            while(ColumbPos < ColumnCount)
            {
                strcat(ImportStatement, SIM.ColumbNames[ColumbPos]);
                strcat(ImportStatement, "=");
                strcat(ImportStatement, SIM.SingleRowData[ColumbPos]);
                if(ColumbPos < ColumnCount - 1)
                {
                    strcat(ImportStatement, ",");
                }
                ColumbPos++;
            }

            strcat(ImportStatement, " ");
            strcat(ImportStatement, WhereStatement);

//            ErrorMessage(NULL, ImportStatement); // Debug code
        }

        //----------------------------------------------------------------------------------------------

        if(ImportStatement != NULL)
        {
//            ErrorMessage(NULL, "Importing"); // Debug code

            if(!CI[Client].WPCustDB.DBExecute(ImportStatement, &StatementHandle))
            {
                // FIXME: Need to dump the statement that failed to a seperate log file. (ImportStatement)

                sprintf(StatusText, "(%d) Error in ImportSQLResultsFailed:: DBExecute failed on ImportStatement.", MyServer.ClientID[Client]);
                WriteLog(StatusText);

                WriteLog(ImportStatement); // Debug Code

                FreeSingleRowDataColumbs(&SIM, ColumnCount);
                FreeSingleRowData(&SIM);

                free(ImportStatement);
                ImportStatement = NULL;

                if(SourceHandle) fclose(SourceHandle);
                glErrorCount++;
                return false;
            }

            DBCloseCursor(&StatementHandle);

            free(ImportStatement);
            ImportStatement = NULL;
        }

        ImportStatement = NULL;

        FreeSingleRowDataColumbs(&SIM, ColumnCount);
        //FreeSingleRowData(&SIM); // This memory needs to stay intact untill we are done with the import

        ColumbPos++;

        RowPos++;
    }

    sprintf(StatusText, "(%d) Successfully imported %d client data rows.", MyServer.ClientID[Client], RowPos);
    WriteLog(StatusText);

    FreeSingleRowData(&SIM);

    if(SourceHandle) fclose(SourceHandle);

    return true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

